@@ -50,7 +50,7 @@ namespace bsccs {
 #if __cplusplus >= 201402L
 	using std::make_unique; // Use C++14 version
 #else
-	// Emulate 
+	// Emulate
 	template<typename T, typename ...Args>
 	bsccs::unique_ptr<T> make_unique( Args&& ...args ) {
     	return bsccs::unique_ptr<T>( new T( std::forward<Args>(args)... ) );
@@ -59,17 +59,24 @@ namespace bsccs {
 
 // Internal types
 
-#ifdef DOUBLE_PRECISION
-	typedef double real;
-#else
-	typedef float real;
-#endif
+// #ifdef DOUBLE_PRECISION
+// 	typedef double real;
+// #else
+// 	typedef float real;
+// #endif
+
+// template <typename T>
+// using Vector = std::vector<T>;
+
+// template <typename T>
+// using VectorPtr = bsccs::shared_ptr<Vector<T> >;
 
 typedef std::vector<int> IntVector;
-typedef std::vector<real> RealVector;
 typedef bsccs::shared_ptr<IntVector> IntVectorPtr;
-typedef bsccs::shared_ptr<RealVector> RealVectorPtr;
+
 typedef int64_t IdType;
+typedef double Fp64Type;
+typedef float Fp32Type;
 
 // Output types
 
@@ -139,6 +146,32 @@ enum class ModelType {
 	SIZE_OF_ENUM // Keep at end
 };
 
+enum class AlgorithmType {
+	CCD = 0,
+	MM,
+	SIZE_OF_ENUM // Keep at end
+};
+
+enum class PrecisionType {
+	FP64 = 0,
+	FP32,
+	FP16,
+	SIZE_OF_ENUM // Keep at end
+};
+
+enum class MmBoundType {
+	SHADDOX = 0,
+	METHOD_1,
+	METHOD_2,
+	SIZE_OF_ENUM // Keep at end
+};
+
+enum class DeviceType {
+    CPU = 0,
+    GPU,
+    SIZE_OF_ENUM // Keep at end
+};
+
 enum class SelectorType {
 	DEFAULT,
 	BY_PID,
